# -*- coding: utf-8 -*-
from base64 import urlsafe_b64encode
import rpc
import conf
import random
import string
try:
	import zlib as binascii
except ImportError:
	import binascii


class PutExtra(object):
	params = {}
	mime_type = 'application/octet-stream'
	crc32 = ""
	check_crc = 0


def put(uptoken, key, data, extra=None):
	""" put your data to Qiniu

	If key is None, the server will generate one.
	data may be str or read()able object.
	"""
	fields = {
	}

	if not extra:
		extra = PutExtra()

	if extra.params:
		for k in extra.params:
			fields[k] = str(extra.params[k])

	if extra.check_crc:
		fields["crc32"] = str(extra.crc32)

	if key is not None:
		fields['key'] = key

	fields["token"] = uptoken

	fname = key
	if fname is None:
		fname = _random_str(9)
	elif fname is '':
		fname = 'index.html'
	files = [
		{'filename': fname, 'data': data, 'mime_type': extra.mime_type},
	]
	return rpc.Client(conf.UP_HOST).call_with_multipart("/", fields, files)


def put_file(uptoken, key, localfile, extra=None):
	""" put a file to Qiniu

	If key is None, the server will generate one.
	"""
	if extra is not None and extra.check_crc == 1:
		extra.crc32 = _get_file_crc32(localfile)
	with open(localfile) as f:
		return put(uptoken, key, f, extra)


_BLOCK_SIZE = 1024 * 1024 * 4

def _get_file_crc32(filepath):
	with open(filepath) as f: 
		block = f.read(_BLOCK_SIZE)
		crc = 0
		while len(block) != 0:
			crc = binascii.crc32(block, crc) & 0xFFFFFFFF
			block = f.read(_BLOCK_SIZE)
	return crc


def _random_str(length):
	lib = string.ascii_lowercase
	return ''.join([random.choice(lib) for i in range(0, length)])



# -*- coding: utf-8 -*-
import os
import unittest
import string
import random
import urllib
try:
	import zlib as binascii
except ImportError:
	import binascii
import cStringIO

from qiniu import conf
from qiniu import rs
from qiniu import io

conf.ACCESS_KEY = os.getenv("QINIU_ACCESS_KEY")
conf.SECRET_KEY = os.getenv("QINIU_SECRET_KEY")
bucket_name = os.getenv("QINIU_TEST_BUCKET")

policy = rs.PutPolicy(bucket_name)
extra = io.PutExtra()
extra.mime_type = "text/plain"
extra.params = {'x:a':'a'}

def r(length):
	lib = string.ascii_uppercase
	return ''.join([random.choice(lib) for i in range(0, length)])

class TestUp(unittest.TestCase):
	def test(self):
		def test_put():
			key = "test_%s" % r(9)
			params = "op=3"
			data = "hello bubby!"
			extra.check_crc = 2
			extra.crc32 = binascii.crc32(data) & 0xFFFFFFFF
			ret, err = io.put(policy.token(), key, data, extra)
			assert err is None
			assert ret['key'] == key

		def test_put_same_crc():
			key = "test_%s" % r(9)
			data = "hello bubby!"
			extra.check_crc = 2
			ret, err = io.put(policy.token(), key, data, extra)
			assert err is None
			assert ret['key'] == key

		def test_put_no_key():
			data = r(100)
			extra.check_crc = 0
			ret, err = io.put(policy.token(), key=None, data=data, extra=extra)
			assert err is None
			assert ret['hash'] == ret['key']

		def test_put_quote_key():
			data = r(100)
			key = 'a\\b\\c"你好' + r(9)
			ret, err = io.put(policy.token(), key, data)
			print err
			assert err is None
			assert ret['key'].encode('utf8') == key

			data = r(100)
			key = u'a\\b\\c"你好' + r(9)
			ret, err = io.put(policy.token(), key, data)
			assert err is None
			assert ret['key'] == key

		def test_put_unicode1():
			key = "test_%s" % r(9) + '你好'
			data = key
			ret, err = io.put(policy.token(), key, data, extra)
			assert err is None
			assert ret[u'key'].endswith(u'你好')

		def test_put_unicode2():
			key = "test_%s" % r(9) + '你好'
			data = key
			data = data.decode('utf8')
			ret, err = io.put(policy.token(), key, data)
			assert err is None
			assert ret[u'key'].endswith(u'你好')

		def test_put_unicode3():
			key = "test_%s" % r(9) + '你好'
			data = key
			key = key.decode('utf8')
			ret, err = io.put(policy.token(), key, data)
			assert err is None
			assert ret[u'key'].endswith(u'你好')

		def test_put_unicode4():
			key = "test_%s" % r(9) + '你好'
			data = key
			key = key.decode('utf8')
			data = data.decode('utf8')
			ret, err = io.put(policy.token(), key, data)
			assert err is None
			assert ret[u'key'].endswith(u'你好')

		def test_put_StringIO():
			key = "test_%s" % r(9)
			data = cStringIO.StringIO('hello buddy!')
			ret, err = io.put(policy.token(), key, data)
			assert err is None
			assert ret['key'] == key

		def test_put_urlopen():
			key = "test_%s" % r(9)
			data = urllib.urlopen('http://cheneya.qiniudn.com/hello_jpg')
			ret, err = io.put(policy.token(), key, data)
			assert err is None
			assert ret['key'] == key

		def test_put_no_length():
			class test_reader(object):
				def __init__(self):
					self.data = 'abc'
					self.pos = 0
				def read(self, n=None):
					if n is None or n < 0:
						newpos = len(self.data)
					else:
						newpos = min(self.pos+n, len(self.data))
					r = self.data[self.pos: newpos]
					self.pos = newpos
					return r
			key = "test_%s" % r(9)
			data = test_reader()

			extra.check_crc = 2
			extra.crc32 = binascii.crc32('abc') & 0xFFFFFFFF
			ret, err = io.put(policy.token(), key, data, extra)
			assert err is None
			assert ret['key'] == key

		test_put()
		test_put_same_crc()
		test_put_no_key()
		test_put_quote_key()
		test_put_unicode1()
		test_put_unicode2()
		test_put_unicode3()
		test_put_unicode4()
		test_put_StringIO()
		test_put_urlopen()
		test_put_no_length()

	def test_put_file(self):
		localfile = "%s" % __file__
		key = "test_%s" % r(9)

		extra.check_crc = 1
		ret, err = io.put_file(policy.token(), key, localfile, extra)
		assert err is None
		assert ret['key'] == key

	def test_put_crc_fail(self):
		key = "test_%s" % r(9)
		data = "hello bubby!"
		extra.check_crc = 2
		extra.crc32 = "wrong crc32"
		ret, err = io.put(policy.token(), key, data, extra)
		assert err is not None


class Test_get_file_crc32(unittest.TestCase):
	def test_get_file_crc32(self):
		file_path = '%s' % __file__

		data = None
		with open(file_path) as f:
			data = f.read()
		io._BLOCK_SIZE = 4
		assert binascii.crc32(data) & 0xFFFFFFFF == io._get_file_crc32(file_path)


if __name__ == "__main__":
	unittest.main()



# -*- coding: utf-8 -*-
import os
import urllib

import qiniu.io
import qiniu.rs
import qiniu.conf

pic = "http://cheneya.qiniudn.com/hello_jpg"
key = 'QINIU_UNIT_TEST_PIC'

def setUp():
	qiniu.conf.ACCESS_KEY = os.getenv("QINIU_ACCESS_KEY")
	qiniu.conf.SECRET_KEY = os.getenv("QINIU_SECRET_KEY")
	bucket_name = os.getenv("QINIU_TEST_BUCKET")

	policy = qiniu.rs.PutPolicy(bucket_name)
	uptoken = policy.token()

	f = urllib.urlopen(pic)
	_, err = qiniu.io.put(uptoken, key, f)
	f.close()
	if err is None or err.startswith('file exists'):
		print err
		assert err is None or err.startswith('file exists')



# -*- coding: utf-8 -*-
from urlparse import urlparse
import hmac
from hashlib import sha1
from base64 import urlsafe_b64encode

from .. import rpc
from .. import conf

class Mac(object):
	access = None
	secret = None
	def __init__(self, access=None, secret=None):
		if access is None and secret is None:
			access, secret = conf.ACCESS_KEY, conf.SECRET_KEY
		self.access, self.secret = access, secret

	def __sign(self, data):
		hashed = hmac.new(self.secret, data, sha1)
		return urlsafe_b64encode(hashed.digest())

	def sign(self, data):
		return '%s:%s' % (self.access, self.__sign(data))

	def sign_with_data(self, b):
		data = urlsafe_b64encode(b)
		return '%s:%s:%s' % (self.access, self.__sign(data), data)

	def sign_request(self, path, body, content_type):
		parsedurl = urlparse(path)
		p_query = parsedurl.query
		p_path = parsedurl.path
		data = p_path
		if p_query != "":
			data = ''.join([data, '?', p_query])
		data = ''.join([data, "\n"])

		if body:
			incBody = [
				"application/x-www-form-urlencoded",
			]
			if content_type in incBody:
				data += body

		return '%s:%s' % (self.access, self.__sign(data))


class Client(rpc.Client):
	def __init__(self, host, mac=None):
		if mac is None:
			mac = Mac()
		super(Client, self).__init__(host)
		self.mac = mac

	def round_tripper(self, method, path, body):
		token = self.mac.sign_request(path, body, self._header.get("Content-Type"))
		self.set_header("Authorization", "QBox %s" % token)
		return super(Client, self).round_tripper(method, path, body)



'''
Qiniu Resource Storage SDK for Python
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

For detailed document, please see:
<https://github.com/qiniu/python-sdk/blob/develop/docs/README.md>
'''

# -*- coding: utf-8 -*-
__version__ = '6.1.0'



"""
Modified from standard httplib

1. HTTPConnection can send trunked data.
2. Remove httplib's automatic Content-Length insertion when data is a file-like object.
"""

# -*- coding: utf-8 -*-

import httplib
from httplib import _CS_REQ_STARTED, _CS_REQ_SENT
import string
import os
from array import array

class HTTPConnection(httplib.HTTPConnection):

	def send(self, data, is_chunked=False):
		"""Send `data' to the server."""
		if self.sock is None:
			if self.auto_open:
				self.connect()
			else:
				raise NotConnected()

		if self.debuglevel > 0:
			print "send:", repr(data)
		blocksize = 8192
		if hasattr(data,'read') and not isinstance(data, array):
			if self.debuglevel > 0: print "sendIng a read()able"
			datablock = data.read(blocksize)
			while datablock:
				if self.debuglevel > 0:
					print 'chunked:', is_chunked
				if is_chunked:
					if self.debuglevel > 0: print 'send: with trunked data'
					lenstr = string.upper(hex(len(datablock))[2:])
					self.sock.sendall('%s\r\n%s\r\n' % (lenstr, datablock))
				else:
					self.sock.sendall(datablock)
				datablock = data.read(blocksize)
			if is_chunked:
				self.sock.sendall('0\r\n\r\n')
		else:
			self.sock.sendall(data)


	def _set_content_length(self, body):
		# Set the content-length based on the body.
		thelen = None
		try:
			thelen = str(len(body))
		except (TypeError, AttributeError), te:
			# Don't send a length if this failed
			if self.debuglevel > 0: print "Cannot stat!!"

		if thelen is not None:
			self.putheader('Content-Length', thelen)
			return True
		return False


	def _send_request(self, method, url, body, headers):
		# Honor explicitly requested Host: and Accept-Encoding: headers.
		header_names = dict.fromkeys([k.lower() for k in headers])
		skips = {}
		if 'host' in header_names:
			skips['skip_host'] = 1
		if 'accept-encoding' in header_names:
			skips['skip_accept_encoding'] = 1

		self.putrequest(method, url, **skips)

		is_chunked = False
		if body and header_names.get('Transfer-Encoding') == 'chunked':
			is_chunked = True
		elif body and ('content-length' not in header_names):
			is_chunked = not self._set_content_length(body)
			if is_chunked:
				self.putheader('Transfer-Encoding', 'chunked')
		for hdr, value in headers.iteritems():
			self.putheader(hdr, value)

		self.endheaders(body, is_chunked=is_chunked)


	def endheaders(self, message_body=None, is_chunked=False):
		"""Indicate that the last header line has been sent to the server.

		This method sends the request to the server.  The optional
		message_body argument can be used to pass a message body
		associated with the request.  The message body will be sent in
		the same packet as the message headers if it is string, otherwise it is
		sent as a separate packet.
		"""
		if self.__state == _CS_REQ_STARTED:
			self.__state = _CS_REQ_SENT
		else:
			raise CannotSendHeader()
		self._send_output(message_body, is_chunked=is_chunked)


	def _send_output(self, message_body=None, is_chunked=False):
		"""Send the currently buffered request and clear the buffer.

		Appends an extra \\r\\n to the buffer.
		A message_body may be specified, to be appended to the request.
		"""
		self._buffer.extend(("", ""))
		msg = "\r\n".join(self._buffer)
		del self._buffer[:]
		# If msg and message_body are sent in a single send() call,
		# it will avoid performance problems caused by the interaction
		# between delayed ack and the Nagle algorithm.
		if isinstance(message_body, str):
			msg += message_body
			message_body = None
		self.send(msg)
		if message_body is not None:
			#message_body was not a string (i.e. it is a file) and
			#we must run the risk of Nagle
			self.send(message_body, is_chunked=is_chunked)




# -*- coding: utf-8 -*-
from base64 import urlsafe_b64encode

from ..auth import digest
from .. import conf

class Client(object):
	conn = None
	def __init__(self, mac=None):
		if mac is None:
			mac = digest.Mac()
		self.conn = digest.Client(host=conf.RS_HOST, mac=mac)

	def stat(self, bucket, key):
		return self.conn.call(uri_stat(bucket, key))

	def delete(self, bucket, key):
		return self.conn.call(uri_delete(bucket, key))

	def move(self, bucket_src, key_src, bucket_dest, key_dest):
		return self.conn.call(uri_move(bucket_src, key_src, bucket_dest, key_dest))

	def copy(self, bucket_src, key_src, bucket_dest, key_dest):
		return self.conn.call(uri_copy(bucket_src, key_src, bucket_dest, key_dest))

	def batch(self, ops):
		return self.conn.call_with_form("/batch", dict(op=ops))

	def batch_stat(self, entries):
		ops = []
		for entry in entries:
			ops.append(uri_stat(entry.bucket, entry.key))
		return self.batch(ops)

	def batch_delete(self, entries):
		ops = []
		for entry in entries:
			ops.append(uri_delete(entry.bucket, entry.key))
		return self.batch(ops)

	def batch_move(self, entries):
		ops = []
		for entry in entries:
			ops.append(uri_move(entry.src.bucket, entry.src.key, 
				entry.dest.bucket, entry.dest.key))
		return self.batch(ops)

	def batch_copy(self, entries):
		ops = []
		for entry in entries:
			ops.append(uri_copy(entry.src.bucket, entry.src.key, 
				entry.dest.bucket, entry.dest.key))
		return self.batch(ops)

class EntryPath(object):
	bucket = None
	key = None
	def __init__(self, bucket, key):
		self.bucket = bucket
		self.key = key

class EntryPathPair:
	src = None
	dest = None
	def __init__(self, src, dest):
		self.src = src
		self.dest = dest

def uri_stat(bucket, key):
	return "/stat/%s" % urlsafe_b64encode("%s:%s" % (bucket, key))

def uri_delete(bucket, key):
	return "/delete/%s" % urlsafe_b64encode("%s:%s" % (bucket, key))

def uri_move(bucket_src, key_src, bucket_dest, key_dest):
	src = urlsafe_b64encode("%s:%s" % (bucket_src, key_src))
	dest = urlsafe_b64encode("%s:%s" % (bucket_dest, key_dest))
	return "/move/%s/%s" % (src, dest)

def uri_copy(bucket_src, key_src, bucket_dest, key_dest):
	src = urlsafe_b64encode("%s:%s" % (bucket_src, key_src))
	dest = urlsafe_b64encode("%s:%s" % (bucket_dest, key_dest))
	return "/copy/%s/%s" % (src, dest)



# -*- coding: utf-8 -*-

__all__ = [
	"Client", "EntryPath", "EntryPathPair", "uri_stat", "uri_delete", "uri_move", "uri_copy", 
	"PutPolicy", "GetPolicy", "make_base_url",
]

from .rs import *
from .rs_token import *



# -*- coding: utf-8 -*-

ACCESS_KEY = "<access_key>"
SECRET_KEY = "<secret_key>"

RS_HOST = "rs.qbox.me"
RSF_HOST = "rsf.qbox.me"
UP_HOST = "up.qiniu.com"

from . import __version__
USER_AGENT = "qiniu python-sdk v%s" % __version__



# -*- coding: utf-8 -*-
import httplib_chunk as httplib
import json
import cStringIO
import conf


class Client(object):
	_conn = None
	_header = None

	def __init__(self, host):
		self._conn = httplib.HTTPConnection(host)
		self._header = {}

	def round_tripper(self, method, path, body):
		self._conn.request(method, path, body, self._header)
		resp = self._conn.getresponse()
		return resp

	def call(self, path):
		return self.call_with(path, None)

	def call_with(self, path, body, content_type=None, content_length=None):
		ret = None

		self.set_header("User-Agent", conf.USER_AGENT)
		if content_type is not None:
			self.set_header("Content-Type", content_type)

		if content_length is not None:
			self.set_header("Content-Length", content_length)

		resp = self.round_tripper("POST", path, body)
		try:
			ret = resp.read()
			ret = json.loads(ret)
		except IOError, e:
			return None, e
		except ValueError:
			pass

		if resp.status / 100 != 2:
			err_msg = ret if "error" not in ret else ret["error"]
			detail = resp.getheader("x-log", None)
			if detail is not None:
				err_msg += ", detail:%s" % detail

			return None, err_msg

		return ret, None

	def call_with_multipart(self, path, fields=None, files=None):
		"""
		 *  fields => {key}
		 *  files => [{filename, data, content_type}]
		"""
		content_type, mr = self.encode_multipart_formdata(fields, files)
		return self.call_with(path, mr, content_type, mr.length())

	def call_with_form(self, path, ops):
		"""
		 * ops => {"key": value/list()}
		"""

		body = []
		for i in ops:
			if isinstance(ops[i], (list, tuple)):
				data = ('&%s=' % i).join(ops[i])
			else:
				data = ops[i]

			body.append('%s=%s' % (i, data))
		body = '&'.join(body)

		content_type = "application/x-www-form-urlencoded"
		return self.call_with(path, body, content_type, len(body))

	def set_header(self, field, value):
		self._header[field] = value

	def set_headers(self, headers):
		self._header.update(headers)

	def encode_multipart_formdata(self, fields, files):
		"""
		 *  fields => {key}
		 *  files => [{filename, data, content_type}]
		 *  return content_type, content_length, body
		"""
		if files is None:
			files = []
		if fields is None:
			fields = {}

		readers = []
		BOUNDARY = '----------ThIs_Is_tHe_bouNdaRY_$'
		CRLF = '\r\n'
		L1 = []
		for key in fields:
			L1.append('--' + BOUNDARY)
			L1.append('Content-Disposition: form-data; name="%s"' % key)
			L1.append('')
			L1.append(fields[key])
		b1 = CRLF.join(L1)
		readers.append(b1)

		for file_info in files:
			L = []
			L.append('')
			L.append('--' + BOUNDARY)
			disposition = "Content-Disposition: form-data;"
			filename = _qiniu_escape(file_info.get('filename'))
			L.append('%s name="file"; filename="%s"' % (disposition, filename))
			L.append('Content-Type: %s' % file_info.get('content_type', 'application/octet-stream'))
			L.append('')
			L.append('')
			b2 = CRLF.join(L)
			readers.append(b2)

			data = file_info.get('data')
			readers.append(data)

		L3 = ['', '--' + BOUNDARY + '--', '']
		b3 = CRLF.join(L3)
		readers.append(b3)

		content_type = 'multipart/form-data; boundary=%s' % BOUNDARY
		return content_type, MultiReader(readers)

def _qiniu_escape(s):
	edits = [('\\', '\\\\'), ('\"', '\\\"')]
	for (search, replace) in edits:
		s = s.replace(search, replace)
	return s


class MultiReader(object):
	""" class MultiReader([readers...])

	MultiReader returns a read()able object that's the logical concatenation of
	the provided input readers.  They're read sequentially.
	"""

	def __init__(self, readers):
		self.readers = []
		self.content_length = 0
		self.valid_content_length = True
		for r in readers:
			if hasattr(r, 'read'):
				if self.valid_content_length:
					length = self._get_content_length(r)
					if length is not None:
						self.content_length += length
					else:
						self.valid_content_length = False
			else:
				buf = r
				if not isinstance(buf, basestring):
					buf = str(buf)
				buf = encode_unicode(buf)
				r = cStringIO.StringIO(buf)
				self.content_length += len(buf)
			self.readers.append(r)


	# don't name it __len__, because the length of MultiReader is not alway valid.
	def length(self):
		return self.content_length if self.valid_content_length else None


	def _get_content_length(self, reader):
		data_len = None
		if hasattr(reader, 'seek') and hasattr(reader, 'tell'):
			try:
				reader.seek(0, 2)
				data_len= reader.tell()
				reader.seek(0, 0)
			except OSError:
				# Don't send a length if this failed
				data_len = None
		return data_len

	def read(self, n=-1):
		if n is None or n == -1:
			return ''.join([encode_unicode(r.read()) for r in self.readers])
		else:
			L = []
			while len(self.readers) > 0 and n > 0:
				b = self.readers[0].read(n)
				if len(b) == 0:
					self.readers = self.readers[1:]
				else:
					L.append(encode_unicode(b))
					n -= len(b)
			return ''.join(L)


def encode_unicode(u):
	if isinstance(u, unicode):
		u = u.encode('utf8')
	return u



# -*- coding: utf-8 -*-
import unittest
from qiniu import conf

class TestConfig(unittest.TestCase):
	def test_USER_AGENT(self):
		assert len(conf.USER_AGENT) >= len('qiniu python-sdk')
	
if __name__ == '__main__':
	unittest.main()



# -*- coding: utf-8 -*-
import StringIO
import unittest

from qiniu import rpc
from qiniu import conf

def round_tripper(client, method, path, body):
	pass

class ClsTestClient(rpc.Client):
	def round_tripper(self, method, path, body):
		round_tripper(self, method, path, body)
		return super(ClsTestClient, self).round_tripper(method, path, body)

client = ClsTestClient(conf.RS_HOST)

class TestClient(unittest.TestCase):
	def test_call(self):
		global round_tripper

		def tripper(client, method, path, body):
			self.assertEqual(path, "/hello")
			assert body is None

		round_tripper = tripper
		client.call("/hello")

	def test_call_with(self):
		global round_tripper
		def tripper(client, method, path, body):
			self.assertEqual(body, "body")

		round_tripper = tripper
		client.call_with("/hello", "body")

	def test_call_with_multipart(self):
		global round_tripper
		def tripper(client, method, path, body):
			target_type = "multipart/form-data"
			self.assertTrue(client._header["Content-Type"].startswith(target_type))
			start_index = client._header["Content-Type"].find("boundary")
			boundary = client._header["Content-Type"][start_index + 9: ]
			dispostion = 'Content-Disposition: form-data; name="auth"'
			tpl = "--%s\r\n%s\r\n\r\n%s\r\n--%s--\r\n" % (boundary, dispostion,
					"auth_string", boundary)
			self.assertEqual(len(tpl), client._header["Content-Length"])
			self.assertEqual(len(tpl), body.length())

		round_tripper = tripper
		client.call_with_multipart("/hello", fields={"auth": "auth_string"})

	def test_call_with_form(self):
		global round_tripper
		def tripper(client, method, path, body):
			self.assertEqual(body, "action=a&op=a&op=b")
			target_type = "application/x-www-form-urlencoded"
			self.assertEqual(client._header["Content-Type"], target_type)
			self.assertEqual(client._header["Content-Length"], len(body))

		round_tripper = tripper
		client.call_with_form("/hello", dict(op=["a", "b"], action="a"))


class TestMultiReader(unittest.TestCase):
	def test_multi_reader1(self):
		a = StringIO.StringIO('你好')
		b = StringIO.StringIO('abcdefg')
		c = StringIO.StringIO(u'悲剧')
		mr = rpc.MultiReader([a, b, c])
		data = mr.read()
		assert data.index('悲剧') > data.index('abcdefg')

	def test_multi_reader2(self):
		a = StringIO.StringIO('你好')
		b = StringIO.StringIO('abcdefg')
		c = StringIO.StringIO(u'悲剧')
		mr = rpc.MultiReader([a, b, c])
		data = mr.read(8)
		assert len(data) is 8


def encode_multipart_formdata2(fields, files):
	if files is None:
		files = []
	if fields is None:
		fields = []

	BOUNDARY = '----------ThIs_Is_tHe_bouNdaRY_$'
	CRLF = '\r\n'
	L = []
	for (key, value) in fields:
		L.append('--' + BOUNDARY)
		L.append('Content-Disposition: form-data; name="%s"' % key)
		L.append('')
		L.append(value)
	for (key, filename, value) in files:
		L.append('--' + BOUNDARY)
		disposition = "Content-Disposition: form-data;"
		L.append('%s name="%s"; filename="%s"' % (disposition, key, filename))
		L.append('Content-Type: application/octet-stream')
		L.append('')
		L.append(value)
	L.append('--' + BOUNDARY + '--')
	L.append('')
	body = CRLF.join(L)
	content_type = 'multipart/form-data; boundary=%s' % BOUNDARY
	return content_type, body


class TestEncodeMultipartFormdata(unittest.TestCase):
	def test_encode(self):
		fields = {'a': '1', 'b': '2'}
		files = [
			{
				'filename': 'key1',
				'data': 'data1',
				'mime_type': 'application/octet-stream',
			},
			{
				'filename': 'key2',
				'data': 'data2',
				'mime_type': 'application/octet-stream',
			}
		]
		content_type, mr = rpc.Client('localhost').encode_multipart_formdata(fields, files)
		t, b = encode_multipart_formdata2(
			[('a', '1'), ('b', '2')],
			[('file', 'key1', 'data1'), ('file', 'key2', 'data2')]
		)
		assert t == content_type
		assert len(b) == mr.length()

	def test_unicode(self):
		def test1():
			files = [{'filename': '你好', 'data': '你好', 'mime_type': ''}]
			_, body = rpc.Client('localhost').encode_multipart_formdata(None, files)
			return len(body.read())
		def test2():
			files = [{'filename': u'你好', 'data': '你好', 'mime_type': ''}]
			_, body = rpc.Client('localhost').encode_multipart_formdata(None, files)
			return len(body.read())
		def test3():
			files = [{'filename': '你好', 'data': u'你好', 'mime_type': ''}]
			_, body = rpc.Client('localhost').encode_multipart_formdata(None, files)
			return len(body.read())
		def test4():
			files = [{'filename': u'你好', 'data': u'你好', 'mime_type': ''}]
			_, body = rpc.Client('localhost').encode_multipart_formdata(None, files)
			return len(body.read())

		assert test1() == test2()
		assert test2() == test3()
		assert test3() == test4()


if __name__ == "__main__":
	unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
	from setuptools import setup
except ImportError:    
	from distutils.core import setup 

PACKAGE = 'qiniu'
NAME = 'qiniu'
DESCRIPTION = 'Qiniu Resource Storage SDK for Python 2.X.'
LONG_DESCRIPTION = 'see:\nhttps://github.com/qiniu/python-sdk\n'
AUTHOR = 'Shanghai Qiniu Information Technologies Co., Ltd.'
AUTHOR_EMAIL = 'support@qiniu.com'
MAINTAINER_EMAIL = 'fengliyuan@qiniu.com'
URL = 'https://github.com/qiniu/python-sdk'
VERSION = __import__(PACKAGE).__version__


setup(
	name=NAME,
	version=VERSION,
	description=DESCRIPTION,
	long_description=LONG_DESCRIPTION,
	author=AUTHOR,
	author_email=AUTHOR_EMAIL,
	maintainer_email=MAINTAINER_EMAIL,
	license='MIT',
	url=URL,
	packages=['qiniu', 'qiniu.test', 'qiniu.auth', 'qiniu.rs', 'qiniu.rs.test'],
	platforms='any',
	classifiers=[
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Operating System :: OS Independent',
		'Programming Language :: Python',
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 2.6',
		'Programming Language :: Python :: 2.7',
		'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
		'Topic :: Software Development :: Libraries :: Python Modules'
	],
	test_suite = 'nose.collector'
)



# -*- coding:utf-8 -*-
import unittest
import os
from qiniu import fop

pic = "http://cheneya.qiniudn.com/hello_jpg"

class TestFop(unittest.TestCase):
	def test_exif(self):
		ie = fop.Exif()
		ret = ie.make_request(pic)
		self.assertEqual(ret, "%s?exif" % pic)

	def test_imageView(self):
		iv = fop.ImageView()
		iv.height = 100
		ret = iv.make_request(pic)
		self.assertEqual(ret, "%s?imageView/1/h/100" % pic)
		
		iv.quality = 20
		iv.format = "png"
		ret = iv.make_request(pic)
		self.assertEqual(ret, "%s?imageView/1/h/100/q/20/format/png" % pic)

	def test_imageInfo(self):
		ii = fop.ImageInfo()
		ret = ii.make_request(pic)
		self.assertEqual(ret, "%s?imageInfo" % pic)


if __name__ == '__main__':
	unittest.main()



# -*- coding: utf-8 -*-
import unittest
from qiniu import rsf
from qiniu import conf

import os
conf.ACCESS_KEY = os.getenv("QINIU_ACCESS_KEY")
conf.SECRET_KEY = os.getenv("QINIU_SECRET_KEY")
bucket_name = os.getenv("QINIU_TEST_BUCKET")

class TestRsf(unittest.TestCase):
	def test_list_prefix(self):
		c = rsf.Client()
		ret, err = c.list_prefix(bucket_name, limit = 1)
		self.assertEqual(err is rsf.EOF or err is None, True)
		assert len(ret.get('items')) == 1


if __name__ == "__main__":
	unittest.main()
